/**
 * These overrides let you run desktop within in browser
 */

/**
 * The BrowserSharedObject uses session cookies to store data
 */
window.BrowserSharedObject = function() {

  this.data = {};
  if (!window.localStorage) {
    this.initCookies();
    this.flush = this.saveCookieValues;
  }
  else {
    this.initStorage();
    this.flush = this.saveStorageValues;
  }
}

window.BrowserSharedObject.prototype = {
  flush: function () {}
  
, initCookies: function () {
    var cookies = document.cookie;

    if (cookies) {
      var parts = cookies.split(/;\s?/);
      _.each(parts, function(part) {
        var pair = part.split('=');
        this.data[decodeURIComponent(pair[0])] = decodeURIComponent(pair[1]);
      }, this);
    }
  }

, saveCookieValues: function () {
    _.each(this.data, function(value, key) {
      document.cookie = encodeURIComponent(key) + '=' + encodeURIComponent(value);
    }, this);
  }
  
, initStorage: function () {
    var key;
    for (var i=0,ii=localStorage.length; i < ii; i++) {
      key = localStorage.key(i);
      this.data[key] = localStorage[key];
    }
  }
  
, saveStorageValues: function () {
    _.each(this.data, function (value, key) {
      try {
        localStorage[key] = value;
      } catch (e) {
        // SOMETHING BAD LIKE EXCEEDING THE QUOTA HAPPENED
      }
    });
  }
};

window.browserLocalOverrides = {
  readLocalFile: function (path) {
    var request = new XMLHttpRequest();
    request.open('GET', path, false);
    request.send(null);
    if (request.status == 200) {
      return request.responseText;
    }
    throw new Error('Failed XHR request to: ' + path);
  }
  ,
  hostenv: 'browser'
  ,
  parseHostEnvOptions: function() {
    window.hostenvOptions.withAdl  = false;
  }
  ,
  setUserAgent: function() {

  }
  ,
  initDockIcon: function() {
    window.dockIcon = {
      setCount: function() {

      }
    }
  }
  ,
  /**
   * instead of using ondominitialize, we provide a public function to call
   */
  connectFrame: function(frame) {
   window.browserConnectRemote = _.bind(function() {
      this.connectRemote();
      window.$r = this.getRemoteWindow();
    }, this);
  }
}

window.BrowserBridgeOverrides = function(_private) {
  _.extend(this, {
  _get: yam.desktop.local.bridge.get,
  _invoke: yam.desktop.local.bridge.invoke,
  _addEventListener: yam.desktop.local.bridge.addEventListener,
  _removeEventListener: yam.desktop.local.bridge.removeEventListener,
  get: function(objPath, start) {
    if (_.isString(objPath) && objPath.indexOf('air') == 0) {
      if (objPath == 'air.Event.COMPLETE') {
        return 'load';
      }
      else if(objPath == 'air.Screen.mainScreen.visibleBounds') {
        return {
          top: window.screen.top,
          left: window.screen.left,
          right: window.screen.width,
          bottom: window.screen.height
        };
      }

      return '';
    }

    return this._get.apply(this, arguments);
  }
  ,
  invoke: function(objPath) {
    if (_.isString(objPath) && objPath.indexOf('air') == 0) {
      console.log('ignoring bridge.invoke: ' + objPath);
      return '';
    }
    return this._invoke.apply(this, arguments);
  }
  ,
  parseApplicationDescriptor: function (key) {
    return ''
  }
  ,
  addEventListener: function(targetObjPath, eventName, fn) {
    if (_.isString(targetObjPath) && targetObjPath.indexOf('air') == 0) {
      console.log('ignoring bridge.addEventListener: ' + targetObjPath);
    }
    else {
     var realEventName = eventName;
     if (eventName.indexOf('.') > 0 ) {
      realEventName = this.get(eventName);
     }
     if (realEventName == 'deactivate') {
       setTimeout(fn, 1);
     }
     else {
       return this._addEventListener.apply(this, arguments);
     }
    }
  }
  ,
  removeEventListener: function(targetObjPath, eventName, fn) {
    if (_.isString(targetObjPath) &&  targetObjPath.indexOf('air') == 0) {
      console.log('ignoring bridge.removeEventListener: ' + targetObjPath);

    }
    else {
      this._removeEventListener.apply(this, arguments);
    }

  }
  ,
  setStartOnBoot: function() {

  }
  ,
  setWindowCoords: function (coords, windowId) {
    var domWindow = this._getWindow(windowId);

    if (!isNaN(Number(coords.height))
    && !isNaN(Number(coords.width))
    && !isNaN(Number(coords.x))
    && !isNaN(Number(coords.y))) {
      domWindow.moveTo(coords.x, coords.y);
      domWindow.resizeTo(coords.width, coords.height);
    }
  }
  ,
  getWindowCoords: function (windowId) {
    var domWindow = windowId ? window.instanceManager.get(windowId) : window;
    return {
      x: domWindow.screenX,
      y: domWindow.screenY,
      width: domWindow.outerWidth,
      height: domWindow.outerHeight
    };
  },
  createWindow: function(options, windowOptions) {

    var url = options.url;

    url =  url.replace('app:/', '/air/app/');

    var windowId = window.instanceManager.nextId();
    var width = options.coords.width || 100;
    var height = options.coords.height || 100;
    var html = window.open(url, windowId, 'width=' + width + ',height=' + height + ',toolbar=no,menubar=no,location=no,resizable=yes,scrollbars=no,status=no');

    html.addEventListener('load', function() {
      html.windowId = windowId;
    }, false);
    
    if (!window.popups) {
      window.popups = [];
    }
    window.popups.push(html);

    window.instanceManager.put(windowId, html);
    return windowId;

  },
  getSharedObject: (function() {
    var so = new window.BrowserSharedObject();
    return function () {
      return so;
    };
  })()
    ,

    getLogFileProperties: function() {
      return {
        path: '/path/to/log/file',
        url: '/url/to/log/file',
        userAgent: navigator.userAgent,
        data: '<data>'
      };
    }
    ,
    /**
     * open the url in the user's browser
     */
    openUrl: function(url) {
      window.open(url);
    }
});

}


window.nativeWindow = window;
window.active = true;
window.activate = function() {};

/**
 * close all the spawned windows when this window closes
 */
window.onbeforeunload = function() {
  _.each(window.popups, function(win) {
    if (!win.closed) {
      win.close();
    }
  });
}