/**
 * error logger for desktop.
 * this file has no dependencies on yamjs or any other yammer libs
 * 
 * on Mac, file is located /Users/<user>/Library/Preferences/<appid>/Local Store/logs
 * NOTE: this file is loaded by console.nodebug.js
 */
(function() {

var convertExceptionToString = function(evt) {
  var props = {};
  var sb = [evt.exceptionValue];
  if (evt.stackTrace && evt.stackTrace.length) {
    var trace = evt.stackTrace[0];
    sb.push('sourceURL: ' + trace.sourceURL,
      'line: ' + trace.line,
      'functionName: ' + trace.functionName
    );
  }
  return sb.join('\n');
}

var logFile = function (filename){
  var storageDir = air.File.applicationStorageDirectory;

  var ensureDir = function (dir){
    var targetDir = storageDir.resolvePath('logs');
    if (!targetDir.isDirectory){
      // just throw if this fails
      targetDir.createDirectory();
    }
  }
  ensureDir('logs');
  return storageDir.resolvePath('logs/' + filename);
}


var ErrorLogger = function() {
    try {
      var nl = air.File.lineEnding;
      this._setConsole();
      this._listen();
      this._rotate();
      this._file = logFile('log.txt');
      this._write(nl + 'Yammer Desktop ' + this.parseApplicationDescriptor('version') + ' --- ' + this._timestamp() + nl);
    } catch (e) {
      air.trace("[logger] could not create logs.txt: ", e);
    }
}
ErrorLogger.prototype = {
  MAX_FILES: 5

  , _file: null
  , _showErrorReport: false

  , init: function () {

  }
  , parseApplicationDescriptor: function (key) {
      var domParser = new DOMParser(),
      appDescriptor =  air.NativeApplication.nativeApplication.applicationDescriptor;

      return domParser.parseFromString(appDescriptor, 'text/xml').getElementsByTagName(key)[0].textContent;
    }
  , _setConsole: function () {
    var self = this;

    var _log = function (level) {
      return function () {
        var args = Array.prototype.slice.call(arguments);
        args.unshift(level);
        self._log.apply(self, args);
      };
    };

    window.console = {
      log: _log('log')
      , debug: _log('debug')
      , info:  _log('info')
      , dir:   _log('dir')
      , error: _log('error')
      , xhr: _log('xhr')
      , isFileLog: true
    }

    this._setXhr();
  }

  , _rotate: function (){
    var file = logFile('log.txt');

    if (file.exists){
      file.moveTo(logFile('log-' + new Date().getTime() + '.txt'));
    }
    this._trimFiles();
  }
  
  , _trimFiles: function () {
    var files = logFile('').getDirectoryListing();

    // Alpha sorting but unless we get into the year 3000 we will be ok
    files.sort(function (a, b){
      return a.nativePath < b.nativePath;
    });

    var trimFiles = files.splice(this.MAX_FILES - 1);
    for (var i = 0, ii = trimFiles.length; i < ii; ++i){
      try {
        air.trace('Trimming file: ', trimFiles[i]);
        trimFiles[i].deleteFile();
      } catch (e) {
        // Couldn't delete file? oh well... maybe next time
        air.trace("Couldn't trim file: " + trimFiles[i] + ": " + e.message);
      }
    }
  }
  
  , _log: function () {
    var nl = air.File.lineEnding
      , logs = ''
      , args = Array.prototype.slice.call(arguments)
      , arg;
      
    logs += '[' + args.shift().toString() + '] ';
    logs += this._timestamp() + ' -- ' + window.location.href + nl;
    while (args.length) {
      arg = args.shift();
      if (arg) {
        logs += ' ';
        if (typeof arg == 'object') {
          try {
            logs += JSON.stringify(arg);
          } catch (e) {
            logs += this._flatten(arg);
          }
        } else {
          logs += arg.toString();
        }
      }
    }
    this._write(logs);
  }
  
  /** _flatten outputs first layer of object as key-value string  */
  , _flatten: function (obj) {
    var vals = []
      , val;
    for (var key in obj) {
      val = obj[key];
      if (val) {
        vals.push(key + ': ' + (typeof val == 'function' ? '[function]' : val.toString()));
      }
    }
    return vals.length ? '{' + vals.join(',') + '}' : obj.toString();
  }
  
  , _write: function (text) {
    try {
      var stream = new air.FileStream()
        , self = this;
      stream.open(this._file, air.FileMode.APPEND);
      stream.writeUTFBytes(text + air.File.lineEnding);
    } catch (e) {
       try {
          air.trace("[logger] write failed");
       }
       catch (will_loop_forever_if_not_caught) { }
    } finally {
       try {
          stream.close();
       }
       catch (will_loop_forever_if_not_caught) { }
    }
  }
  
  , _timestamp: function () {
    return new Date().toUTCString();
  }
  
  , _listen: function () {
    window.htmlLoader.addEventListener("uncaughtScriptException", function (e) {
      try {
        console.error(convertExceptionToString(e));
      }
      catch (ex) {
        try {
          air.trace("Failed to log error:" + ex);
        }
        catch (will_loop_forever_if_not_caught) { }
      }
      
    });
  }

  , _setXhr: function () {
    var open = window.XMLHttpRequest.prototype.open;
    if (window.XMLHttpRequest && window.XMLHttpRequest.prototype) {
      window.XMLHttpRequest.prototype.oOpen = window.XMLHttpRequest.prototype.open;
      window.XMLHttpRequest.prototype.open = function () {
        console.xhr.apply(null, arguments);
        this.oOpen.apply(this, arguments);
      };
    }

  }
};

if (window.opener) {
  var win = window.opener;
    window.htmlLoader.addEventListener("uncaughtScriptException", function (e) {
      try {
        win.console.error(convertExceptionToString(e));
      }
      catch (ex) {
        try {
          air.trace("Failed to log error:" + ex);
        }
        catch (will_loop_forever_if_not_caught) { }
      }
    });
}
else {
  window.errorLogger = new ErrorLogger();
}

  
})();
